#!/usr/bin/python3
#
# A simple audio player with changing the playback speed
# Copyright (C) 2014 Katsuya Matsubara
#
import sys
from os import path
import time
import gi
gi.require_version('Gst', '1.0')
from gi.repository import GObject, Gst

if not len(sys.argv) == 4:
    print('Usage: %s filename rate delay' % sys.argv[0])
    quit()

GObject.threads_init()
Gst.init(None)
filename = path.join(path.dirname(path.abspath(__file__)), sys.argv[1])

class Fastplay:
    def __init__(self):        
        self.mainloop = GObject.MainLoop()
        self.pipeline = Gst.Pipeline()
        self.bus = self.pipeline.get_bus()
        self.bus.add_signal_watch()
        self.bus.connect('message::eos', self.on_eos)
        self.bus.connect('message::error', self.on_error)

        self.src = Gst.ElementFactory.make('filesrc', None)
        self.dec = Gst.ElementFactory.make('decodebin', None)
        self.tempo = Gst.ElementFactory.make('scaletempo', None)
        self.sink =  Gst.ElementFactory.make('alsasink', None)

        self.pipeline.add(self.src)
        self.pipeline.add(self.dec)
        self.pipeline.add(self.tempo)
        self.pipeline.add(self.sink)

        self.src.set_property('location', filename)
        self.sink.set_property('device', 'hw:1,0')

        self.dec.connect('pad-added', self.on_pad_added)

        self.src.link(self.dec)
        self.tempo.link(self.sink)

    def run(self):
        self.pipeline.set_state(Gst.State.PLAYING)
        time.sleep(float(sys.argv[3]))
        self.pipeline.seek(float(sys.argv[2]),
                           Gst.Format.TIME,
                           Gst.SeekFlags.FLUSH,
                           Gst.SeekType.SET,
                           self.pipeline.query_position(Gst.Format.TIME)[0],
                           Gst.SeekType.NONE, 0)
        self.mainloop.run()

    def kill(self):
        self.pipeline.set_state(Gst.State.NULL)
        self.mainloop.quit()

    def on_pad_added(self, element, pad):
        string = pad.query_caps(None).to_string()
        if string.startswith('audio/'):
            pad.link(self.tempo.get_static_pad('sink'))

    def on_eos(self, bus, msg):
        self.kill()

    def on_error(self, bus, msg):
        print('on_error():', msg.parse_error())
        self.kill()


fastplay = Fastplay()
fastplay.run()
